MODULE sbcice_lim
   !!======================================================================
   !!                       ***  MODULE  sbcice_lim  ***
   !! Surface module :  update surface ocean boundary condition over ice
   !!                   covered area using LIM sea-ice model
   !! Sea-Ice model  :  LIM 3.0 Sea ice model time-stepping
   !!======================================================================
   !! History :  9.0   !  06-12  (M. Vancoppenolle) Original code
   !!            9.0   !  06-06  (G. Madec)  Surface module from icestp.F90
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3' :                                  LIM 3.0 sea-ice model
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   sbc_ice_lim  : sea-ice model time-stepping and
   !!                  update ocean sbc over ice-covered area
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE c1d             ! 1d configuration
   USE dom_oce         ! ocean space and time domain
   USE par_ice         ! sea-ice parameters
   USE ice
   USE iceini
   USE ice_oce         ! ice variables
   USE dom_ice
   USE cpl_oce

   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbc_ice         ! Surface boundary condition: ice   fields
   USE sbcblk_core     ! Surface boundary condition: CORE bulk
   USE sbcblk_clio     ! Surface boundary condition: CLIO bulk
   USE albedo

   USE daymod          ! day calendar
   USE phycst          ! Define parameters for the routines
   USE eosbn2          ! equation of state
   USE limdyn          ! Ice dynamics
   USE limtrp          ! Ice transport
   USE limthd          ! Ice thermodynamics
   USE limitd_th       ! Thermodynamics on ice thickness distribution 
   USE limitd_me       ! Mechanics on ice thickness distribution
   USE limsbc          ! sea surface boundary condition
   USE limdia          ! Ice diagnostics
   USE limwri          ! Ice outputs
   USE limrst          ! Ice restarts
   USE limupdate       ! update of global variables
   USE limvar          ! Ice variables switch

   USE lbclnk
   USE iom             ! I/O manager library
   USE in_out_manager  ! I/O manager
   USE prtctl          ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC sbc_ice_lim  ! routine called by sbcmod.F90
   
   CHARACTER(len=1) ::   cl_grid = 'C'     ! type of grid used in ice dynamics

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2006) 
   !! $ Id: $
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE sbc_ice_lim( kt, kblk )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE sbc_ice_lim  ***
      !!                   
      !! ** Purpose :   update the ocean surface boundary condition via the 
      !!                Louvain la Neuve Sea Ice Model time stepping 
      !!
      !! ** Method  :   ice model time stepping
      !!              - call the ice dynamics routine 
      !!              - call the ice advection/diffusion routine 
      !!              - call the ice thermodynamics routine 
      !!              - call the routine that computes mass and 
      !!                heat fluxes at the ice/ocean interface
      !!              - save the outputs 
      !!              - save the outputs for restart when necessary
      !!
      !! ** Action  : - time evolution of the LIM sea-ice model
      !!              - update all sbc variables below sea-ice:
      !!                utau, vtau, qns , qsr, emp , emps
      !!---------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt      ! ocean time step
      INTEGER, INTENT(in) ::   kblk    ! type of bulk (=3 CLIO, =4 CORE)
      !!
      INTEGER  ::   ji, jj, jk, jl     ! dummy loop indices
      INTEGER  ::   indx               ! indexes for ice points
      INTEGER  ::   numaltests         ! number of alert tests (max 20)
      INTEGER  ::   alert_id           ! number of the current alert
      REAL(wp) ::   ztmelts            ! ice layer melting point
      INTEGER , DIMENSION(20) ::  numal                     ! number of alerts positive
      CHARACTER (len=30), DIMENSION(20) ::   alname      ! name of alert
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   alb_ice_os   ! albedo of the ice under overcast sky
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   alb_ice_cs   ! albedo of ice under clear sky
      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'sbc_ice_lim : update ocean surface boudary condition' 
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~   via Louvain la Neuve Ice Model (LIM) time stepping'

         CALL ice_init

         !+++++
         indx = 12
         jiindx = 44
         jjindx = 140
         WRITE(numout,*) ' The debugging point is : jiindx : ',jiindx,  &
                                                  ' jjindx : ',jjindx
         !+++++

      ENDIF

      IF( MOD( kt-1, nn_fsbc ) == 0 ) THEN
         !
         ! ... mean surface ocean current at ice dynamics point
         !     C-grid dynamics :  U- & V-points as the ocean
         u_oce(:,:) = ssu_m(:,:) * tmu(:,:)
         v_oce(:,:) = ssv_m(:,:) * tmv(:,:)
         !
         CALL lbc_lnk( u_oce, 'U', -1. )   ! U-point
         CALL lbc_lnk( v_oce, 'V', -1. )   ! V-point

         ! ... masked sea surface freezing temperature [Kelvin] (set to rt0 over land)
         t_bo(:,:) = tfreez( sss_m ) +  rt0 


         ! ... ice albedo
         CALL albedo_ice( t_su, ht_i, ht_s, alb_ice_cs, alb_ice_os )

         ! ... Sea-ice surface boundary conditions output from bulk formulae :
         !     - utaui_ice  ! surface ice stress i-component (I-point)   [N/m2]
         !     - vtaui_ice  ! surface ice stress j-component (I-point)   [N/m2]
         !     - qns_ice    ! non solar heat flux over ice   (T-point)   [W/m2]
         !     - qsr_ice    !     solar heat flux over ice   (T-point)   [W/m2]
         !     - qla_ice    ! latent    heat flux over ice   (T-point)   [W/m2]
         !     - dqns_ice   ! non solar heat sensistivity    (T-point)   [W/m2]
         !     - dqla_ice   ! latent    heat sensistivity    (T-point)   [W/m2]
         !     - tprecip    ! total precipitation            (T-point)   [Kg/m2/s]
         !     - sprecip    ! solid precipitation            (T-point)   [Kg/m2/s]
         !     - fr1_i0     ! 1sr fraction of qsr penetration in ice     [%]
         !     - fr2_i0     ! 2nd fraction of qsr penetration in ice     [%]
         !
         SELECT CASE( kblk )
         CASE( 3 )           ! CLIO bulk formulation
            CALL blk_ice_clio( t_su , u_ice , v_ice   , alb_ice_cs, alb_ice_os,            &
               &                             utaui_ice, vtaui_ice , qns_ice   , qsr_ice,   &
               &                             qla_ice  , dqns_ice  , dqla_ice  ,            &
               &                             tprecip  , sprecip   ,                        &
               &                             fr1_i0   , fr2_i0    , cl_grid  )
         CASE( 4 )           ! CORE bulk formulation
            CALL blk_ice_core( t_su , u_ice , v_ice   , alb_ice_cs,                      &
               &                             utaui_ice, vtaui_ice , qns_ice , qsr_ice,   &
               &                             qla_ice  , dqns_ice  , dqla_ice,            &
               &                             tprecip  , sprecip   ,                      &
               &                             fr1_i0   , fr2_i0    , cl_grid  )
         END SELECT

         IF(ln_ctl) THEN         ! print mean trends (used for debugging)
            CALL prt_ctl_info( 'Ice Forcings ' )
            CALL prt_ctl( tab2d_1=tprecip  ,clinfo1=' sbc_ice_lim: precip   : ' )
            CALL prt_ctl( tab2d_1=utaui_ice,clinfo1=' sbc_ice_lim: utaui_ice: ', tab2d_2=vtaui_ice, clinfo2=' vtaui_ice: ' )
            CALL prt_ctl( tab2d_1=sst_m    ,clinfo1=' sbc_ice_lim: sst      : ', tab2d_2=sss_m    , clinfo2=' sss      : ' )
            CALL prt_ctl( tab2d_1=u_oce    ,clinfo1=' sbc_ice_lim: u_io     : ', tab2d_2=v_oce    , clinfo2=' v_io     : ' )
            CALL prt_ctl( tab2d_1=frld     ,clinfo1=' sbc_ice_lim: frld   1 : ' )
            !
            DO jl = 1, jpl
               CALL prt_ctl_info('* - category number ', ivar1=jl)
               CALL prt_ctl(tab3d_1=t_su    , clinfo1=' sbc_ice_lim: t_su      : ', kdim=jl)
               CALL prt_ctl(tab3d_1=qsr_ice , clinfo1=' sbc_ice_lim: qsr_ice   : ', kdim=jl)
               CALL prt_ctl(tab3d_1=qns_ice , clinfo1=' sbc_ice_lim: qns_ice   : ', kdim=jl)
               CALL prt_ctl(tab3d_1=dqns_ice, clinfo1=' sbc_ice_lim: dqns_ice  : ', kdim=jl)
               CALL prt_ctl(tab3d_1=qla_ice , clinfo1=' sbc_ice_lim: qla_ice   : ', kdim=jl)
               CALL prt_ctl(tab3d_1=dqla_ice, clinfo1=' sbc_ice_lim: dqla_ice  : ', kdim=jl)
            END DO
            !
         ENDIF

         !------------------------------------------------
         ! Store old values of ice model global variables
         !------------------------------------------------

         old_a_i(:,:,:)   = a_i(:,:,:)     ! ice area
         old_e_i(:,:,:,:) = e_i(:,:,:,:)   ! ice thermal energy
         old_v_i(:,:,:)   = v_i(:,:,:)     ! ice volume
         old_v_s(:,:,:)   = v_s(:,:,:)     ! snow volume 
         old_e_s(:,:,:,:) = e_s(:,:,:,:)   ! snow thermal energy
         old_smv_i(:,:,:) = smv_i(:,:,:)   ! salt content
         old_oa_i(:,:,:)  = oa_i(:,:,:)    ! areal age content

         d_a_i_thd(:,:,:)   = 0.e0 ; d_a_i_trp(:,:,:)   = 0.e0
         d_v_i_thd(:,:,:)   = 0.e0 ; d_v_i_trp(:,:,:)   = 0.e0
         d_e_i_thd(:,:,:,:) = 0.e0 ; d_e_i_trp(:,:,:,:) = 0.e0
         d_v_s_thd(:,:,:)   = 0.e0 ; d_v_s_trp(:,:,:)   = 0.e0
         d_e_s_thd(:,:,:,:) = 0.e0 ; d_e_s_trp(:,:,:,:) = 0.e0
         d_smv_i_thd(:,:,:) = 0.e0 ; d_smv_i_trp(:,:,:) = 0.e0
         d_oa_i_thd(:,:,:)  = 0.e0 ; d_oa_i_trp(:,:,:)  = 0.e0

         fseqv(:,:)     = 0.e0
         fsbri(:,:)     = 0.e0     ; fsalt_res(:,:) = 0.e0
         fsalt_rpo(:,:) = 0.e0
         fhmec(:,:)     = 0.e0     ; fhbri(:,:)     = 0.e0
         fmmec(:,:)     = 0.e0     ; fheat_res(:,:) = 0.e0
         fheat_rpo(:,:) = 0.e0     ; focea2D(:,:)   = 0.e0
         fsup2D(:,:)    = 0.e0

         diag_sni_gr(:,:) = 0.e0   ; diag_lat_gr(:,:) = 0.e0
         diag_bot_gr(:,:) = 0.e0   ; diag_dyn_gr(:,:) = 0.e0
         diag_bot_me(:,:) = 0.e0   ; diag_sur_me(:,:) = 0.e0

         ! dynamical invariants
         delta_i(:,:) = 0.e0
         divu_i(:,:)  = 0.e0
         shear_i(:,:) = 0.e0

         !----------------!
         ! Ice model step !
         !----------------!
                         numit = numit + nn_fsbc
                         CALL lim_rst_opn( kt )          ! Open Ice restart file
                         !+++++
                         WRITE(numout,*) ' - Beginning the time step - '
                         CALL lim_inst_state(jiindx,jjindx,1)
                         WRITE(numout,*) ' '
                         !+++++
         !---------------------|
         ! Dynamical processes |       
         !---------------------|
         IF( .NOT. lk_c1d ) THEN                         ! Ice dynamics & transport (not in 1D case)
                         CALL lim_dyn                         ! Ice dynamics    ( rheology/dynamics )
                         CALL lim_trp                         ! Ice transport   ( Advection/diffusion )
                         CALL lim_var_agg(1)                  ! aggregate categories, requested
                         CALL lim_var_glo2eqv                 ! equivalent variables, requested for rafting
                         !+++++
                         WRITE(numout,*) ' - After ice dynamics and transport '
                         CALL lim_inst_state( jiindx, jjindx, 1 )
                         WRITE(numout,*)
                         WRITE(numout,*) ' Mechanical Check ************** '
                         WRITE(numout,*) ' Check what means ice divergence '
                         WRITE(numout,*) ' Total ice concentration ', at_i (jiindx,jjindx)
                         WRITE(numout,*) ' Total lead fraction     ', ato_i(jiindx,jjindx)
                         WRITE(numout,*) ' Sum of both             ', ato_i(jiindx,jjindx) + at_i(jiindx,jjindx)
                         WRITE(numout,*) ' Sum of both minus 1     ', ato_i(jiindx,jjindx) + at_i(jiindx,jjindx) - 1.00
                         !+++++
                         CALL lim_itd_me                      ! Mechanical redistribution ! (ridging/rafting)
         ENDIF
         !--------------------|
         ! Ice thermodynamics |
         !--------------------|
                         CALL lim_var_glo2eqv            ! equivalent variables
                         CALL lim_var_agg(1)             ! aggregate ice categories
                         CALL lim_var_bv                 ! bulk brine volume (diag)
                         CALL lim_thd                    ! Ice thermodynamics 
                         oa_i(:,:,:) = oa_i(:,:,:)   &
                            &         + a_i(:,:,:)   &
                            &         * rdt_ice      &
                            &         / 86400.00         ! Ice natural aging
                         CALL lim_var_glo2eqv            ! except info message that follows, 
                         !                               ! this CALL is maybe not necessary
                         !+++++
                         WRITE(numout,*) ' - After ice thermodynamics '
                         CALL lim_inst_state(jiindx,jjindx,1)
                         !+++++
                         CALL lim_itd_th                 !  Remap ice categories, lateral accretion  !
         !-------------------------|
         ! Global variables update |
         !-------------------------|
                         CALL lim_var_agg(1)             ! requested by limupdate
                         CALL lim_update                 ! Global variables update
                         CALL lim_var_glo2eqv            ! equivalent variables (outputs)
                         CALL lim_var_agg(2)             ! aggregate ice thickness categories
                         !+++++
                         IF(ln_nicep) THEN
                            WRITE(numout,*) ' - Final ice state after lim_update '
                            CALL lim_inst_state(jiindx,jjindx,2)
                            WRITE(numout,*) ' '
                         ENDIF
                         !+++++
         !--------------------------------------|
         ! Fluxes of mass and heat to the ocean |
         !--------------------------------------|
                         CALL lim_sbc( kt )              ! Ice/Ocean Mass & Heat fluxes 
                         !+++++
                         WRITE(numout,*) ' - Final ice state after lim_flx    '
                         CALL lim_inst_state(jiindx,jjindx,3)
                         WRITE(numout,*) ' '
                         !+++++
         !-------------------------|
         ! Diagnostics and outputs |
         !-------------------------|
         IF( MOD( kt+nn_fsbc-1, ninfo ) == 0 .OR. ntmoy == 1 )   &
          &              CALL lim_dia                    ! Ice Diagnostics 
                         CALL lim_wri      ( 1  )        ! Ice outputs 
         IF( lrst_ice )  CALL lim_rst_write( kt )        ! Ice restart file 
                         CALL lim_var_glo2eqv
         !
         !-------------------------------|
         ! Alerts in case of model crash |
         !-------------------------------|

         numaltests = 10
         numal(:) = 0

         ! Alert if incompatible volume and concentration
         alert_id = 2 ! reference number of this alert
         alname(alert_id) = ' Incompat vol and con         ' ! name of the alert

         DO jl = 1, jpl
            DO jj = 1, jpj
               DO ji = 1, jpi
                     IF ((v_i(ji,jj,jl).NE.0.0).AND.(a_i(ji,jj,jl).EQ.0.0)) THEN
                        WRITE(numout,*) ' ALERTE 2 '
                        WRITE(numout,*) ' Incompatible volume and concentration '
                        WRITE(numout,*) ' at_i     ', at_i(ji,jj)
                        WRITE(numout,*) ' Point - category', ji, jj, jl
                        WRITE(numout,*)
                        WRITE(numout,*) ' a_i *** a_i_old ', a_i(ji,jj,jl), old_a_i(ji,jj,jl)
                        WRITE(numout,*) ' v_i *** v_i_old ', v_i(ji,jj,jl), old_v_i(ji,jj,jl)
                        WRITE(numout,*) ' d_a_i_thd/trp   ', d_a_i_thd(ji,jj,jl), d_a_i_trp(ji,jj,jl)
                        WRITE(numout,*) ' d_v_i_thd/trp   ', d_v_i_thd(ji,jj,jl), d_v_i_trp(ji,jj,jl)
                        numal(alert_id) = numal(alert_id) + 1
                     ENDIF
               END DO
            END DO
         END DO

         ! Alerte if very thick ice
         alert_id = 3 ! reference number of this alert
         alname(alert_id) = ' Very thick ice               ' ! name of the alert
         jl = jpl 
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF (ht_i(ji,jj,jl) .GT. 50.0 ) THEN
                  WRITE(numout,*) ' ALERTE 3 '
                  WRITE(numout,*) ' Very thick ice '
                  CALL lim_inst_state(ji,jj,2)
                  WRITE(numout,*) ' '
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO

         ! Alert if very fast ice
         alert_id = 4 ! reference number of this alert
         alname(alert_id) = ' Very fast ice               ' ! name of the alert
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( ( ABS( u_ice(ji,jj) ) .GT. 0.50) .OR. &
                      ( ABS( v_ice(ji,jj) ) .GT. 0.50) ) .AND. &
                    ( at_i(ji,jj) .GT. 0.0 ) ) THEN
                  WRITE(numout,*) ' ALERTE 4 '
                  WRITE(numout,*) ' Very fast ice '
                  CALL lim_inst_state(ji,jj,1)
                  WRITE(numout,*) ' ice strength             : ', strength(ji,jj)
                  WRITE(numout,*) ' oceanic stress utau      : ', utau(ji,jj) 
                  WRITE(numout,*) ' oceanic stress vtau      : ', vtau(ji,jj)
                  WRITE(numout,*) ' sea-ice stress utaui_ice : ', utaui_ice(ji,jj) 
                  WRITE(numout,*) ' sea-ice stress vtaui_ice : ', vtaui_ice(ji,jj)
                  WRITE(numout,*) ' oceanic speed u          : ', u_oce(ji,jj)
                  WRITE(numout,*) ' oceanic speed v          : ', v_oce(ji,jj)
                  WRITE(numout,*) ' sst                      : ', sst_m(ji,jj)
                  WRITE(numout,*) ' sss                      : ', sss_m(ji,jj)
                  WRITE(numout,*) 
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO

         ! Alert if there is ice on continents
         alert_id = 6 ! reference number of this alert
         alname(alert_id) = ' Ice on continents           ' ! name of the alert
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( tms(ji,jj) .LE. 0.0 ) .AND. ( at_i(ji,jj) .GT. 0.0 ) ) THEN 
                  WRITE(numout,*) ' ALERTE 6 '
                  WRITE(numout,*) ' Ice on continents '
                  CALL lim_inst_state(ji,jj,1)
                  WRITE(numout,*) ' masks s, u, v        : ', tms(ji,jj), &
                                                              tmu(ji,jj), &
                                                              tmv(ji,jj) 
                  WRITE(numout,*) ' sst                  : ', sst_m(ji,jj)
                  WRITE(numout,*) ' sss                  : ', sss_m(ji,jj)
                  WRITE(numout,*) ' at_i(ji,jj)          : ', at_i(ji,jj)
                  WRITE(numout,*) ' v_ice(ji,jj)         : ', v_ice(ji,jj)
                  WRITE(numout,*) ' v_ice(ji,jj-1)       : ', v_ice(ji,jj-1)
                  WRITE(numout,*) ' u_ice(ji-1,jj)       : ', u_ice(ji-1,jj)
                  WRITE(numout,*) ' u_ice(ji,jj)         : ', v_ice(ji,jj)

                  numal(alert_id) = numal(alert_id) + 1

               ENDIF
            END DO
         END DO

         ! Alert if very fresh ice
         alert_id = 7 ! reference number of this alert
         alname(alert_id) = ' Very fresh ice               ' ! name of the alert
         DO jl = 1, jpl
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( ( ABS( sm_i(ji,jj,jl) ) .LT. 0.50) .OR. &
                      ( ABS( sm_i(ji,jj,jl) ) .LT. 0.50) ) .AND. &
                    ( a_i(ji,jj,jl) .GT. 0.0 ) ) THEN
!                 WRITE(numout,*) ' ALERTE 7 '
!                 WRITE(numout,*) ' Very fresh ice '
!                 CALL lim_inst_state(ji,jj,1)
!                 WRITE(numout,*) ' sst                  : ', sst_m(ji,jj)
!                 WRITE(numout,*) ' sss                  : ', sss_m(ji,jj)
!                 WRITE(numout,*) ' s_i_newice           : ', s_i_newice(ji,jj,1:jpl)
!                 WRITE(numout,*) 
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO
         END DO

         ! Alert if too old ice
         alert_id = 9 ! reference number of this alert
         alname(alert_id) = ' Very old   ice               ' ! name of the alert
         DO jl = 1, jpl
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( ( ABS( o_i(ji,jj,jl) ) .GT. rdt_ice ) .OR. &
                      ( ABS( o_i(ji,jj,jl) ) .LT. 0.00) ) .AND. &
                    ( a_i(ji,jj,jl) .GT. 0.0 ) ) THEN
                  WRITE(numout,*) ' ALERTE 9 '
                  WRITE(numout,*) ' Wrong ice age '
                  CALL lim_inst_state(ji,jj,1)
                  WRITE(numout,*) 
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO

         END DO

         ! Alert on salt flux
         alert_id = 5 ! reference number of this alert
         alname(alert_id) = ' High salt flux               ' ! name of the alert
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF (ABS(emps(ji,jj)).gt.1.0e-2) THEN
               WRITE(numout,*) ' ALERTE 5 '
               WRITE(numout,*) ' High salt flux '
               CALL lim_inst_state(ji,jj,3)
               WRITE(numout,*) ' '
               DO jl = 1, jpl
                  WRITE(numout,*) ' Category no: ', jl
                  WRITE(numout,*) ' a_i        : ', a_i(ji,jj,jl)     , &
                                  ' old_a_i    : ', old_a_i(ji,jj,jl)   
                  WRITE(numout,*) ' d_a_i_trp  : ', d_a_i_trp(ji,jj,jl) , &
                                  ' d_a_i_thd  : ', d_a_i_thd(ji,jj,jl) 
                  WRITE(numout,*) ' v_i        : ', v_i(ji,jj,jl)    , &
                                  ' old_v_i    : ', old_v_i(ji,jj,jl)   
                  WRITE(numout,*) ' d_v_i_trp  : ', d_v_i_trp(ji,jj,jl) , &
                                  ' d_v_i_thd  : ', d_v_i_thd(ji,jj,jl) 
                  WRITE(numout,*) ' '
                  END DO
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO

         ! Alert if qns very big
         alert_id = 8 ! reference number of this alert
         alname(alert_id) = ' qns very big             ' ! name of the alert
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( ABS( qns(ji,jj) ) .GT. 1500.0 ) .AND. &
                    ( at_i(ji,jj) .GT. 0.0 ) )  THEN

                  WRITE(numout,*) ' ALERTE 8 '
                  WRITE(numout,*) ' ji, jj    : ', ji, jj
                  WRITE(numout,*) ' qns       : ', qns(ji,jj)
                  WRITE(numout,*) ' Very high non-solar heat flux '
                  WRITE(numout,*) ' sst       : ', sst_m(ji,jj)
                  WRITE(numout,*) ' sss       : ', sss_m(ji,jj)
                  WRITE(numout,*) ' qcmif     : ', qcmif(jiindx,jjindx)
                  WRITE(numout,*) ' qldif     : ', qldif(jiindx,jjindx)
                  WRITE(numout,*) ' qcmif     : ', qcmif(jiindx,jjindx) / rdt_ice
                  WRITE(numout,*) ' qldif     : ', qldif(jiindx,jjindx) / rdt_ice
                  WRITE(numout,*) ' qfvbq     : ', qfvbq(jiindx,jjindx)
                  WRITE(numout,*) ' qdtcn     : ', qdtcn(jiindx,jjindx)
                  WRITE(numout,*) ' qfvbq / dt: ', qfvbq(jiindx,jjindx) / rdt_ice
                  WRITE(numout,*) ' qdtcn / dt: ', qdtcn(jiindx,jjindx) / rdt_ice
                  WRITE(numout,*) ' fdtcn     : ', fdtcn(jiindx,jjindx) 
                  WRITE(numout,*) ' fhmec     : ', fhmec(jiindx,jjindx) 
                  WRITE(numout,*) ' fheat_rpo : ', fheat_rpo(jiindx,jjindx) 
                  WRITE(numout,*) ' fheat_res : ', fheat_res(jiindx,jjindx) 
                  WRITE(numout,*) ' fhbri     : ', fhbri(jiindx,jjindx) 

                  CALL lim_inst_state(ji,jj,2)
                  numal(alert_id) = numal(alert_id) + 1

               ENDIF
            END DO
         END DO
         !+++++
 
         ! Alert if very warm ice
         alert_id = 10 ! reference number of this alert
         alname(alert_id) = ' Very warm ice                ' ! name of the alert
         numal(alert_id) = 0
         DO jl = 1, jpl
         DO jk = 1, nlay_i
         DO jj = 1, jpj
            DO ji = 1, jpi
               ztmelts    =  -tmut*s_i(ji,jj,jk,jl) + rtt
               IF ( ( t_i(ji,jj,jk,jl) .GE. ztmelts) .AND. &
                    ( v_i(ji,jj,jl)   .GT. 1.0e-6)   .AND. &
                    ( a_i(ji,jj,jl) .GT. 0.0     )  ) THEN
                  WRITE(numout,*) ' ALERTE 10 '
                  WRITE(numout,*) ' ji, jj, jk, jl : ', ji, jj, jk, jl
                  WRITE(numout,*) ' Very warm ice '
                  WRITE(numout,*) ' t_i : ', t_i(ji,jj,jk,jl)
                  WRITE(numout,*) ' e_i : ', e_i(ji,jj,jk,jl)
                  WRITE(numout,*) ' s_i : ', s_i(ji,jj,jk,jl)
                  WRITE(numout,*) ' ztmelts : ', ztmelts
                  numal(alert_id) = numal(alert_id) + 1
               ENDIF
            END DO
         END DO
         END DO
         END DO

         alert_id = 1 ! reference number of this alert
         alname(alert_id) = ' Il n''y a pas d''alerte 1       ' ! name of the alert
         WRITE(numout,*)
         WRITE(numout,*) ' All alerts at the end of ice model '
         DO alert_id = 1, numaltests
            WRITE(numout,*) alert_id, alname(alert_id)//' : ', numal(alert_id), ' times ! '
         END DO
         !
      ENDIF  ! End sea-ice coupling
      !
   END SUBROUTINE sbc_ice_lim


   SUBROUTINE lim_inst_state( ki, kj, kn )
      !!-----------------------------------------------------------------------
      !!                   ***  ROUTINE lim_inst_state *** 
      !!                 
      !! ** Purpose :   Writes global ice state on the (i,j) point 
      !!                in ocean.ouput 
      !!                3 possibilities exist 
      !!                n = 1 -> simple ice state
      !!                n = 2 -> exhaustive state
      !!                n = 3 -> ice/ocean salt fluxes
      !!
      !! ** input   :   point coordinates (i,j) 
      !!                n : number of the option
      !!-------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   ki, kj, kn     ! ocean time-step index
      INTEGER :: jl
      !!-------------------------------------------------------------------

      !----------------
      !  Simple state
      !----------------

      IF ( kn .EQ. 1 ) THEN
         WRITE(numout,*) ' lim_inst_state - Point : ',ki,kj
         WRITE(numout,*) ' ~~~~~~~~~~~~~~ '
         WRITE(numout,*) ' Simple state '
         WRITE(numout,*) ' masks s,u,v   : ', tms(ki,kj), tmu(ki,kj), tmv(ki,kj)
         WRITE(numout,*) ' lat - long    : ', gphit(ki,kj), glamt(ki,kj)
         WRITE(numout,*) ' Time step     : ', numit
         WRITE(numout,*) ' - Ice drift   '
         WRITE(numout,*) '   ~~~~~~~~~~~ '
         WRITE(numout,*) ' u_ice(i-1,j)  : ', u_ice(ki-1,kj)
         WRITE(numout,*) ' u_ice(i  ,j)  : ', u_ice(ki,kj)
         WRITE(numout,*) ' v_ice(i  ,j-1): ', v_ice(ki,kj-1)
         WRITE(numout,*) ' v_ice(i  ,j)  : ', v_ice(ki,kj)
         WRITE(numout,*) ' strength      : ', strength(ki,kj)
         WRITE(numout,*)
         WRITE(numout,*) ' - Cell values '
         WRITE(numout,*) '   ~~~~~~~~~~~ '
         WRITE(numout,*) ' cell area     : ', area(ki,kj)
         WRITE(numout,*) ' at_i          : ', at_i(ki,kj)       
         WRITE(numout,*) ' vt_i          : ', vt_i(ki,kj)       
         WRITE(numout,*) ' vt_s          : ', vt_s(ki,kj)       
         DO jl = 1, jpl
            WRITE(numout,*) ' - Category (', jl,')'
            WRITE(numout,*) ' a_i           : ', a_i(ki,kj,jl)
            WRITE(numout,*) ' ht_i          : ', ht_i(ki,kj,jl)
            WRITE(numout,*) ' ht_s          : ', ht_s(ki,kj,jl)
            WRITE(numout,*) ' v_i           : ', v_i(ki,kj,jl)
            WRITE(numout,*) ' v_s           : ', v_s(ki,kj,jl)
            WRITE(numout,*) ' e_s           : ', e_s(ki,kj,1,jl)/1.0e9
            WRITE(numout,*) ' e_i           : ', e_i(ki,kj,1:nlay_i,jl)/1.0e9
            WRITE(numout,*) ' t_su          : ', t_su(ki,kj,jl)
            WRITE(numout,*) ' t_snow        : ', t_s(ki,kj,1,jl)
            WRITE(numout,*) ' t_i           : ', t_i(ki,kj,1:nlay_i,jl)
            WRITE(numout,*) ' sm_i          : ', sm_i(ki,kj,jl)
            WRITE(numout,*) ' smv_i         : ', smv_i(ki,kj,jl)
            WRITE(numout,*)
            WRITE(numout,*) ' Pathological case : ', patho_case(ki,kj,jl)
         END DO
      ENDIF

      !--------------------
      !  Exhaustive state
      !--------------------

      IF ( kn .EQ. 2 ) THEN
         WRITE(numout,*) ' lim_inst_state - Point : ',ki,kj
         WRITE(numout,*) ' ~~~~~~~~~~~~~~ '
         WRITE(numout,*) ' Exhaustive state '
         WRITE(numout,*) ' lat - long ', gphit(ki,kj), glamt(ki,kj)
         WRITE(numout,*) ' Time step ', numit
         WRITE(numout,*) 
         WRITE(numout,*) ' - Cell values '
         WRITE(numout,*) '   ~~~~~~~~~~~ '
         WRITE(numout,*) ' cell area     : ', area(ki,kj)
         WRITE(numout,*) ' at_i          : ', at_i(ki,kj)       
         WRITE(numout,*) ' vt_i          : ', vt_i(ki,kj)       
         WRITE(numout,*) ' vt_s          : ', vt_s(ki,kj)       
         WRITE(numout,*) ' u_ice(i-1,j)  : ', u_ice(ki-1,kj)
         WRITE(numout,*) ' u_ice(i  ,j)  : ', u_ice(ki,kj)
         WRITE(numout,*) ' v_ice(i  ,j-1): ', v_ice(ki,kj-1)
         WRITE(numout,*) ' v_ice(i  ,j)  : ', v_ice(ki,kj)
         WRITE(numout,*) ' strength      : ', strength(ki,kj)
         WRITE(numout,*) ' d_u_ice_dyn   : ', d_u_ice_dyn(ki,kj), ' d_v_ice_dyn   : ', d_v_ice_dyn(ki,kj)
         WRITE(numout,*) ' old_u_ice     : ', old_u_ice(ki,kj)  , ' old_v_ice     : ', old_v_ice(ki,kj)  
         WRITE(numout,*)

         DO jl = 1, jpl
              WRITE(numout,*) ' - Category (',jl,')'
              WRITE(numout,*) '   ~~~~~~~~         ' 
              WRITE(numout,*) ' ht_i       : ', ht_i(ki,kj,jl)             , ' ht_s       : ', ht_s(ki,kj,jl)
              WRITE(numout,*) ' t_i        : ', t_i(ki,kj,1:nlay_i,jl)
              WRITE(numout,*) ' t_su       : ', t_su(ki,kj,jl)             , ' t_s        : ', t_s(ki,kj,1,jl)
              WRITE(numout,*) ' sm_i       : ', sm_i(ki,kj,jl)             , ' o_i        : ', o_i(ki,kj,jl)
              WRITE(numout,*) ' a_i        : ', a_i(ki,kj,jl)              , ' old_a_i    : ', old_a_i(ki,kj,jl)   
              WRITE(numout,*) ' d_a_i_trp  : ', d_a_i_trp(ki,kj,jl)        , ' d_a_i_thd  : ', d_a_i_thd(ki,kj,jl) 
              WRITE(numout,*) ' v_i        : ', v_i(ki,kj,jl)              , ' old_v_i    : ', old_v_i(ki,kj,jl)   
              WRITE(numout,*) ' d_v_i_trp  : ', d_v_i_trp(ki,kj,jl)        , ' d_v_i_thd  : ', d_v_i_thd(ki,kj,jl) 
              WRITE(numout,*) ' v_s        : ', v_s(ki,kj,jl)              , ' old_v_s    : ', old_v_s(ki,kj,jl)  
              WRITE(numout,*) ' d_v_s_trp  : ', d_v_s_trp(ki,kj,jl)        , ' d_v_s_thd  : ', d_v_s_thd(ki,kj,jl)
              WRITE(numout,*) ' e_i1       : ', e_i(ki,kj,1,jl)/1.0e9      , ' old_ei1    : ', old_e_i(ki,kj,1,jl)/1.0e9 
              WRITE(numout,*) ' de_i1_trp  : ', d_e_i_trp(ki,kj,1,jl)/1.0e9, ' de_i1_thd  : ', d_e_i_thd(ki,kj,1,jl)/1.0e9
              WRITE(numout,*) ' e_i2       : ', e_i(ki,kj,2,jl)/1.0e9      , ' old_ei2    : ', old_e_i(ki,kj,2,jl)/1.0e9  
              WRITE(numout,*) ' de_i2_trp  : ', d_e_i_trp(ki,kj,2,jl)/1.0e9, ' de_i2_thd  : ', d_e_i_thd(ki,kj,2,jl)/1.0e9
              WRITE(numout,*) ' e_snow     : ', e_s(ki,kj,1,jl)            , ' old_e_snow : ', old_e_s(ki,kj,1,jl) 
              WRITE(numout,*) ' d_e_s_trp  : ', d_e_s_trp(ki,kj,1,jl)      , ' d_e_s_thd  : ', d_e_s_thd(ki,kj,1,jl)
              WRITE(numout,*) ' smv_i      : ', smv_i(ki,kj,jl)            , ' old_smv_i  : ', old_smv_i(ki,kj,jl)   
              WRITE(numout,*) ' d_smv_i_trp: ', d_smv_i_trp(ki,kj,jl)      , ' d_smv_i_thd: ', d_smv_i_thd(ki,kj,jl) 
              WRITE(numout,*) ' oa_i       : ', oa_i(ki,kj,jl)             , ' old_oa_i   : ', old_oa_i(ki,kj,jl)
              WRITE(numout,*) ' d_oa_i_trp : ', d_oa_i_trp(ki,kj,jl)       , ' d_oa_i_thd : ', d_oa_i_thd(ki,kj,jl)
              WRITE(numout,*) ' Path. case : ', patho_case(ki,kj,jl)
        END DO !jl

        WRITE(numout,*)
        WRITE(numout,*) ' - Heat / FW fluxes '
        WRITE(numout,*) '   ~~~~~~~~~~~~~~~~ '
!       WRITE(numout,*) ' fsbri      : ', fsbri(ki,kj)
!       WRITE(numout,*) ' fseqv      : ', fseqv(ki,kj)
!       WRITE(numout,*) ' fsalt_res  : ', fsalt_res(ki,kj)
        WRITE(numout,*) ' fmmec      : ', fmmec(ki,kj)
        WRITE(numout,*) ' fhmec      : ', fhmec(ki,kj)
        WRITE(numout,*) ' fhbri      : ', fhbri(ki,kj)
        WRITE(numout,*) ' fheat_rpo  : ', fheat_rpo(ki,kj)
        WRITE(numout,*) 
        WRITE(numout,*) ' sst        : ', sst_m(ki,kj)  
        WRITE(numout,*) ' sss        : ', sss_m(ki,kj)  
        WRITE(numout,*) 
        WRITE(numout,*) ' - Stresses '
        WRITE(numout,*) '   ~~~~~~~~ '
        WRITE(numout,*) ' utaui_ice  : ', utaui_ice(ki,kj) 
        WRITE(numout,*) ' vtaui_ice  : ', vtaui_ice(ki,kj)
        WRITE(numout,*) ' utau       : ', utau(ki,kj) 
        WRITE(numout,*) ' vtau       : ', vtau(ki,kj)
        WRITE(numout,*) ' oc. vel. u : ', u_oce(ki,kj)
        WRITE(numout,*) ' oc. vel. v : ', v_oce(ki,kj)
     ENDIF

     !---------------------
     ! Salt / heat fluxes
     !---------------------

     IF ( kn .EQ. 3 ) THEN
        WRITE(numout,*) ' lim_inst_state - Point : ',ki,kj
        WRITE(numout,*) ' ~~~~~~~~~~~~~~ '
        WRITE(numout,*) ' - Salt / Heat Fluxes '
        WRITE(numout,*) '   ~~~~~~~~~~~~~~~~ '
        WRITE(numout,*) ' lat - long ', gphit(ki,kj), glamt(ki,kj)
        WRITE(numout,*) ' Time step ', numit
        WRITE(numout,*)
        WRITE(numout,*) ' - Heat fluxes at bottom interface ***'
        WRITE(numout,*) ' qsr        : ', qsr(ki,kj)
        WRITE(numout,*) ' qns        : ', qns(ki,kj)
        WRITE(numout,*)
        WRITE(numout,*) ' - Salt fluxes at bottom interface ***'
        WRITE(numout,*) ' emps       : ', emps(ki,kj)
        WRITE(numout,*) ' emp        : ', emp(ki,kj)
        WRITE(numout,*) ' fsbri      : ', fsbri(ki,kj)
        WRITE(numout,*) ' fseqv      : ', fseqv(ki,kj)
        WRITE(numout,*) ' fsalt_res  : ', fsalt_res(ki,kj)
        WRITE(numout,*) ' fsalt_rpo  : ', fsalt_rpo(ki,kj)
        WRITE(numout,*) ' - Heat fluxes at bottom interface ***'
        WRITE(numout,*) ' fheat_res  : ', fheat_res(ki,kj)
        WRITE(numout,*)
        WRITE(numout,*) ' - Momentum fluxes '
        WRITE(numout,*) ' utau      : ', utau(ki,kj) 
        WRITE(numout,*) ' vtau      : ', vtau(ki,kj)
      ENDIF

   END SUBROUTINE lim_inst_state

#else
   !!----------------------------------------------------------------------
   !!   Default option           Dummy module      NO LIM 3.0 sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE sbc_ice_lim ( kt, kblk )     ! Dummy routine
      WRITE(*,*) 'sbc_ice_lim: You should not have seen this print! error?', kt, kblk
   END SUBROUTINE sbc_ice_lim
#endif

   !!======================================================================
END MODULE sbcice_lim
